{****************************************************************************}
{*                                                                          *}
{*                                 MAKECHAO                                 *}
{*                                                                          *}
{*              Programm zur Erzeugung von 'symmetrischem Chaos'            *}
{*                          als Muster fr MAGIC-3D                         *}
{*              Version 1.0 (c) Rolf Sander & Martin Simeth 1994            *}
{*                                                                          *}
{* Die Funktionsparameter a, b, c und d knnen in gewissen Grenzen frei     *}
{* gewhlt werden, um verschiedene Muster zu erzeugen. Die Auflsung wird   *}
{* durch PatternxSize und PatternySize bestimmt. Groe Werte knnen zu      *}
{* extrem hohen Rechenzeiten fhren!                                        *}
{*                                                                          *}
{****************************************************************************}

{$R-} {kein range-check}
{$S-} {kein stack-check}
{$I-} {kein I/O-check}
{$D-} {keine Debug-Information}
{$V-} {relaxed var-string-check}
{$B-} {short-circuit boolean evaluation}
{$E+} {Coprozessor emulieren, falls nicht vorhanden}
{$N+} {Coprozessor benutzen}

program MAKECHAO;

uses
  crt,      {fr ClrScr, ReadKey}
  IOFunct;  {Input- und Outputfunktionen}

const
  a               = 1.89;       {Funktionsparameter}
  b               = -1.3;       {Funktionsparameter}
  c               = 0.17;       {Funktionsparameter}
  d               = -0.79;      {Funktionsparameter}
  OutputFileName  = 'Chaos';
  PaletteFileName = 'Rainbow';
  PatternxSize    = 100;         {Breite des Musters}
  PatternySize    = 100;         {Hhe des Musters}

  Imax      = 200*PatternxSize*PatternySize;
  Pi        = 3.14159265;
  xx : Real = 0.377; {Zufallszahl}
  yy : Real = 0.875; {Zufallszahl}

type
  MagicLineType    = array[0 .. PatternxSize] of Byte;
  ColorPaletteType = array[0..255,1..3] of Byte;
  FileOfCharType   = File Of Char; {Bug in Pascal! Typ ohne Leerzeichen}
  ChaosArrayType   = array[0..PatternxSize-1,0..PatternySize-1] of Integer;

var
  blue               : integer;
  ChaosArray         : ChaosArrayType;
  ColorPalette       : ColorPaletteType;
  green              : integer;
  i                  : longint;
  ix                 : longint;
  iy                 : longint;
  MagicLine          : MagicLineType;
  OutputID           : FileOfCharType;      {ID des Musterfiles}
  PaletteID          : text;                {ID des Farbpalettenfiles}
  Percent            : string;
  red                : integer;
  x                  : word;            {x-Position im Bild}
  xnew               : Real;
  y                  : word;            {y-Position im Bild}
  ynew               : Real;

Label
  EndOfProg;

{****************************************************************************}
{**********************          OpenOutput          ************************}
{****************************************************************************}

Procedure OpenOutput;
var
  i         : Integer;
  Character : Char;
  Text      : String;

Begin
  Character := Char(10);
  Assign(OutputID,OutputFileName+'.PCX');
  Rewrite(OutputID);
  Write(OutputID,Character);
  Character := Char(5);
  Write(OutputID,Character);
  Character := Char(1);
  Write(OutputID,Character);
  Character := Char(8);                           {BitPerPixel}
  Write(OutputID,Character);
  Character := Char(0);
  Write(OutputID,Character);
  Write(OutputID,Character);
  Write(OutputID,Character);
  Write(OutputID,Character);
  Character:= Char((PatternxSize-1) and 255);
  Write(OutputID,Character);
  Character :=  Char((PatternxSize-1) shr 8);
  Write(OutputID,Character);
  Character :=  Char((PatternySize-1) and 255);
  Write(OutputID,Character);
  Character :=  Char((PatternySize-1) shr 8);
  Write(OutputID,Character);
  Character :=  Char(PatternxSize and 255);
  Write(OutputID,Character);
  Character :=  Char(PatternxSize shr 8);
  Write(OutputID,Character);
  Character :=  Char(PatternySize and 255);
  Write(OutputID,Character);
  Character :=  Char(PatternySize shr 8);
  Write(OutputID,Character);

  For i := 0 To 15 Do
  Begin
    Write(OutputID,Char(Colorpalette[i,1]));
    Write(OutputID,Char(Colorpalette[i,2]));
    Write(OutputID,Char(Colorpalette[i,3]));
  end;
  Character := Char(0);
  Write(OutputID,Character);
  Character := Char(1);                              {NPlanes}
  Write(OutputID,Character);
  Character :=  Char(PatternxSize and 255);          {BytesPerLine and 255}
  Write(OutputID,Character);
  Character :=  Char(PatternxSize shr 8);            {BytesPerLine shr 8}
  Write(OutputID,Character);
  Character := Char(1);
  Write(OutputID,Character);
  Character := Char(0);
  Write(OutputID,Character);
  Text := 'MAKECHAO Version 1.0  (c) Rolf Sander & Martin Simeth 1994';
  For i := 1 to 58 Do
  Begin
    Write(OutputID, Text[i]);
  end;

end;


{****************************************************************************}
{*********************           WritePCXLine        ************************}
{****************************************************************************}

Procedure WritePCXLine;
var
  x         : Integer;
  Duplicate : Integer;
  Character : Char;
  Value     : Byte;
  Percent   : String;

Begin
  x := 0;
  while x<=PatternxSize-1 Do
  Begin
    Duplicate := 0;
    Value:=MagicLine[x+Duplicate];
    Duplicate:=Duplicate+1;
    While ((x+Duplicate)<=(PatternxSize-1)) and (Duplicate<63)
      and (Value=MagicLine[Duplicate+x]) Do
    Begin
      Duplicate:=Duplicate+1;
    end;
    If (Duplicate>1) or (Value>=192) Then
    Begin
      Character:=Char(192+Duplicate);
      Write(OutputID,Character);
    end;
      Write(OutputID, Char(Value));
    x:=x+Duplicate;
  end;
end;

{****************************************************************************}
{**********************         CloseOutput          ************************}
{****************************************************************************}

Procedure CloseOutput;
var
  Character : Char;
Begin
  Character := Char(12);
  Write(OutputID,Character);
  For i:=0 To 255 Do
  Begin
    Write(OutputID,Char(Colorpalette[i,1]));
    Write(OutputID,Char(Colorpalette[i,2]));
    Write(OutputID,Char(Colorpalette[i,3]));
  end;
  close(OutputID);
end;

{****************************************************************************}
{*********************              main              ***********************}
{****************************************************************************}

Begin
  setscreenptr;
  clrscr;
  PrintBox(20,10,60,16,Normal,Framedouble);
  Printxy(22,12,'   MAKECHAO erzeugt '+OutputFileName+'.PCX',normal);
  Printxy(22,14,'          Prozent',normal);

  For ix := 0 To PatternxSize-1 Do
  begin
    For iy := 0 To PatternySize-1 Do
    begin
      ChaosArray[ix,iy] := 0;
    end;
  end;

{****************************************************************************}

  For i := 1 To Imax Do
  Begin
    {es mu gelten: xnew(xx)+1=xnew(xx+1) und ynew(yy)+1=ynew(yy+1)}
    xnew := a*sin(2*pi*xx) + b*sin(2*pi*xx)*cos(2*pi*yy)
          + c*cos(4*pi*xx) + d*sin(6*pi*yy)*cos(4*pi*xx) + xx;
    ynew := a*sin(2*pi*yy) - b*sin(2*pi*yy)*cos(2*pi*xx)
          + c*cos(4*pi*yy) - d*sin(6*pi*xx)*cos(4*pi*yy) + yy;
    xx := frac(xnew);
    if (xnew<0) then xx:=xx+1;
    yy := frac(ynew);
    if (ynew<0) then yy:=yy+1;
    if (i>100) then {die ersten 100 Punkte werden nicht benutzt}
    begin
      ix := round(int(PatternxSize*xx));
      iy := round(int(PatternySize*yy));
      ChaosArray[ix,iy] := ChaosArray[ix,iy] + 1;
      if (i mod 1000=0) then
      begin
        str(i*100/Imax:6:2,Percent);
        Printxy(25,14,Percent,normal);
      end;
    end;
  end;

{****************************************************************************}

  assign(PaletteID,PaletteFileName+'.PAL'); {Farbpalette im Neopaint-Format}
  Reset(PaletteID);
  Readln(PaletteID);
  Readln(PaletteID);
  Readln(PaletteID);
  For I := 0 to 255 Do
  Begin
    Readln(PaletteID,red,green,blue);
    ColorPalette[i,1] := (4*red) mod 256;
    ColorPalette[i,2] := (4*green) mod 256;
    ColorPalette[i,3] := (4*blue) mod 256;
  End;
  close(PaletteID);

{****************************************************************************}

  OpenOutput;
  for y := 0 to PatternySize-1 do
  Begin
    For x := 0 To PatternxSize-1 do
    Begin
      MagicLine[x] := ChaosArray[x,y] mod 256;
    end;
    WritePCXLine;
  end;
  CloseOutput;
  EndOfProg:
end.

{****************************************************************************}
{****************************     end of file    ****************************}
{****************************************************************************}
